function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var reactNative = require('react-native');

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it;

  if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;
      return function () {
        if (i >= o.length) return {
          done: true
        };
        return {
          done: false,
          value: o[i++]
        };
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  it = o[Symbol.iterator]();
  return it.next.bind(it);
}

var AUTO_SCROLL_AFTER_USER_SCROLL = 6000;

function getRandomString() {
  return Math.random().toString(36).substring(7);
}

var LRC_LINE = /^(\[[0-9]+:[0-9]+(\.[0-9]+)?\])+.*/;
var LRC_TIMESTAMP_WITH_BRACKET = /\[[0-9]+:[0-9]+(\.[0-9]+)?\]/g;
var LRC_TIMESTAMP = /[0-9]+/g;
var parseLrc = (function (lrc) {
  var lrcLineList = [];
  var lineList = lrc.split('\n');

  for (var _iterator = _createForOfIteratorHelperLoose(lineList), _step; !(_step = _iterator()).done;) {
    var line = _step.value;

    if (!LRC_LINE.test(line)) {
      continue;
    }

    var timeStringList = line.match(LRC_TIMESTAMP_WITH_BRACKET);
    var content = line.replace(LRC_TIMESTAMP_WITH_BRACKET, '');

    for (var _iterator2 = _createForOfIteratorHelperLoose(timeStringList), _step2; !(_step2 = _iterator2()).done;) {
      var timeString = _step2.value;

      var _timeString$match = timeString.match(LRC_TIMESTAMP),
          minute = _timeString$match[0],
          second = _timeString$match[1],
          _timeString$match$ = _timeString$match[2],
          millisecond = _timeString$match$ === void 0 ? '0' : _timeString$match$;

      lrcLineList.push({
        id: getRandomString(),
        millisecond: Number.parseInt(minute, 10) * 60 * 1000 + Number.parseInt(second, 10) * 1000 + Number.parseInt(millisecond, 10),
        content: content
      });
    }
  }

  return lrcLineList.sort(function (a, b) {
    return a.millisecond - b.millisecond;
  });
});

var useLrc = (function (lrc) {
  var lrcLineList = React.useMemo(function () {
    return parseLrc(lrc);
  }, [lrc]);
  return lrcLineList;
});

var useCurrentIndex = (function (_ref) {
  var lrcLineList = _ref.lrcLineList,
      currentTime = _ref.currentTime;

  var _useState = React.useState(-1),
      currentIndex = _useState[0],
      setCurrentIndex = _useState[1];

  React.useEffect(function () {
    var length = lrcLineList.length;
    var i = 0;

    for (; i < length; i += 1) {
      var millisecond = lrcLineList[i].millisecond;

      if (currentTime < millisecond) {
        break;
      }
    }

    setCurrentIndex(i - 1);
  }, [currentTime, lrcLineList]);
  return currentIndex;
});

function useThrottle(fn, delay) {
  if (delay === void 0) {
    delay = 300;
  }

  var _useRef = React.useRef({}),
      current = _useRef.current;

  return function () {
    if (!current.timer) {
      current.timer = setTimeout(function () {
        delete current.timer;
      }, delay);

      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      fn.apply(this, args);
    }
  };
}

function useDebounce(fn, delay) {
  if (delay === void 0) {
    delay = 300;
  }

  var _useRef = React.useRef({}),
      current = _useRef.current;

  return function () {
    var _this = this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    if (current.timer) {
      clearTimeout(current.timer);
    }

    current.timer = setTimeout(function () {
      return fn.apply(_this, args);
    }, delay);
  };
}

var useLocalAutoScroll = (function (_ref) {
  var autoScroll = _ref.autoScroll,
      autoScrollAfterUserScroll = _ref.autoScrollAfterUserScroll;

  var _useState = React.useState(autoScroll),
      localAutoScroll = _useState[0],
      setLocalAutoScroll = _useState[1];

  var resetLocalAutoScroll = React.useCallback(function () {
    return setLocalAutoScroll(autoScroll);
  }, [autoScroll]);
  var resetAutoScrollAfterUserScroll = useDebounce(function () {
    return setLocalAutoScroll(autoScroll);
  }, autoScrollAfterUserScroll);
  var onScroll = useThrottle(function () {
    setLocalAutoScroll(false);
    resetAutoScrollAfterUserScroll();
  });
  return {
    localAutoScroll: localAutoScroll,
    resetLocalAutoScroll: resetLocalAutoScroll,
    onScroll: onScroll
  };
});

var Lrc = React__default.forwardRef(function Lrc(_ref, ref) {
  var lrc = _ref.lrc,
      _ref$lineRenderer = _ref.lineRenderer,
      lineRenderer = _ref$lineRenderer === void 0 ? function (_ref2) {
    var content = _ref2.lrcLine.content,
        active = _ref2.active;
    return /*#__PURE__*/React__default.createElement(reactNative.Text, {
      style: {
        textAlign: 'center',
        color: active ? 'green' : '#666'
      }
    }, content);
  } : _ref$lineRenderer,
      _ref$currentTime = _ref.currentTime,
      currentTime = _ref$currentTime === void 0 ? 0 : _ref$currentTime,
      _ref$autoScroll = _ref.autoScroll,
      autoScroll = _ref$autoScroll === void 0 ? true : _ref$autoScroll,
      _ref$lineHeight = _ref.lineHeight,
      lineHeight = _ref$lineHeight === void 0 ? 16 : _ref$lineHeight,
      _ref$activeLineHeight = _ref.activeLineHeight,
      activeLineHeight = _ref$activeLineHeight === void 0 ? lineHeight : _ref$activeLineHeight,
      _ref$autoScrollAfterU = _ref.autoScrollAfterUserScroll,
      autoScrollAfterUserScroll = _ref$autoScrollAfterU === void 0 ? AUTO_SCROLL_AFTER_USER_SCROLL : _ref$autoScrollAfterU,
      onCurrentLineChange = _ref.onCurrentLineChange,
      _ref$height = _ref.height,
      height = _ref$height === void 0 ? 500 : _ref$height,
      style = _ref.style,
      props = _objectWithoutPropertiesLoose(_ref, ["lrc", "lineRenderer", "currentTime", "autoScroll", "lineHeight", "activeLineHeight", "autoScrollAfterUserScroll", "onCurrentLineChange", "height", "style"]);

  var lrcRef = React.useRef(null);
  var lrcLineList = useLrc(lrc);
  var currentIndex = useCurrentIndex({
    lrcLineList: lrcLineList,
    currentTime: currentTime
  });

  var _useLocalAutoScroll = useLocalAutoScroll({
    autoScroll: autoScroll,
    autoScrollAfterUserScroll: autoScrollAfterUserScroll
  }),
      localAutoScroll = _useLocalAutoScroll.localAutoScroll,
      resetLocalAutoScroll = _useLocalAutoScroll.resetLocalAutoScroll,
      onScroll = _useLocalAutoScroll.onScroll;

  React.useEffect(function () {
    if (localAutoScroll) {
      var _lrcRef$current;

      (_lrcRef$current = lrcRef.current) === null || _lrcRef$current === void 0 ? void 0 : _lrcRef$current.scrollTo({
        y: currentIndex * lineHeight || 0,
        animated: true
      });
    }
  }, [currentIndex, localAutoScroll, lineHeight]);
  React.useEffect(function () {
    onCurrentLineChange && onCurrentLineChange({
      index: currentIndex,
      lrcLine: lrcLineList[currentIndex] || null
    });
  }, [lrcLineList, currentIndex, onCurrentLineChange]);
  React.useImperativeHandle(ref, function () {
    return {
      getCurrentLine: function getCurrentLine() {
        return {
          index: currentIndex,
          lrcLine: lrcLineList[currentIndex] || null
        };
      },
      scrollToCurrentLine: function scrollToCurrentLine() {
        var _lrcRef$current2;

        resetLocalAutoScroll();
        (_lrcRef$current2 = lrcRef.current) === null || _lrcRef$current2 === void 0 ? void 0 : _lrcRef$current2.scrollTo({
          y: currentIndex * lineHeight || 0,
          animated: true
        });
      }
    };
  });
  return /*#__PURE__*/React__default.createElement(reactNative.ScrollView, _extends({}, props, {
    ref: lrcRef,
    scrollEventThrottle: 40,
    onScroll: onScroll,
    style: [style, {
      height: height
    }]
  }), /*#__PURE__*/React__default.createElement(reactNative.View, null, autoScroll ? /*#__PURE__*/React__default.createElement(reactNative.View, {
    style: {
      width: '100%',
      height: 0.45 * height
    }
  }) : null, lrcLineList.map(function (lrcLine, index) {
    return /*#__PURE__*/React__default.createElement(reactNative.View, {
      key: lrcLine.id,
      style: {
        height: currentIndex === index ? activeLineHeight : lineHeight
      }
    }, lineRenderer({
      lrcLine: lrcLine,
      index: index,
      active: currentIndex === index
    }));
  }), autoScroll ? /*#__PURE__*/React__default.createElement(reactNative.View, {
    style: {
      width: '100%',
      height: 0.5 * height
    }
  }) : null));
});

exports.Lrc = Lrc;
exports.parseLrc = parseLrc;
exports.useLrc = useLrc;
//# sourceMappingURL=index.js.map
